
/* Copyright (c) 1990 Sonnet Software, Inc., All rights reserved. */
/******************************************************************/

/* %W% last mod on %G% at %U% */
/* Full SCCS path name: (%P%) */

/*******************************************************************

	NAME: dos.c
	
	AUTHOR: K. Schroeder
	
	PURPOSE: Simulates unix calls for MSDOS
	
	CONTENTS:
		getuid() - Get user id.
		getpwuid() - Get password info for user id.
		getpwnam() - Gets password info for user name.
		dos_chdir() - change directory, handle new drive if needed.

	NOTE:
		If PWD_H is defined then assume that these functions
		can be found in a library.
*******************************************************************/

#include "local.h"
#include <stdio.h>
#include <string.h>
#include <time.h>

#ifndef PWD_H
/*******************************************************************

	DESCRIPTION:

	Gets user id.

*******************************************************************/
int
getuid()
{
	return (1);
}

/*******************************************************************

	DESCRIPTION:

	Gets password info for user id.

*******************************************************************/
struct passwd *
getpwuid(uid)
int uid;
{
	return ( (struct passwd *)NULL );
}

/*******************************************************************

	DESCRIPTION:

	Gets password info for user name.

*******************************************************************/
struct passwd *
getpwnam(name)
char *name;
{
	return ( (struct passwd *)NULL );
}

#endif /* Not defined: PWD_H */

/*******************************************************************

	DESCRIPTION:


     tempnam() uses malloc() to get  space  for  the  constructed
     file  name,  and  returns a pointer to this area.  Thus, any
     pointer value returned from tempnam() may serve as an  argu-
     ment  to  free (see malloc(3V)).  If tempnam() cannot return
     the expected  result  for  any  reason,  that  is,  malloc()
     failed,  or  none of the above mentioned attempts to find an
     appropriate directory was successful, a NULL pointer will be
     returned.

*******************************************************************/
#ifndef InstantC
char *
tempnam(dir,pfx)
char *dir;
char *pfx;		/* Must be NULL */
{
	char *base_filename;
	char *filename;
	int len;

	if ( pfx != NULL ) {
		announce_bug("tempnam(0)");
	}
	
	/* Get the file name */
	base_filename = sstmpnam( (char *)NULL );

	len = strlen(base_filename)+1;
	if ( dir == NULL ) { 
		filename = malloc( sizeof(char) * len);
		strcpy(filename,base_filename);
	} else {
		len += strlen(dir)+1;
		filename = malloc( sizeof(char) * len);
		sprintf(filename,"%s/%s",dir,base_filename);
	}
	return(filename);
}
#endif /* InstantC */

char *sstmpnam(s)
char *s;
{
	static unsigned int seed = 0;
	static unsigned int rnd = 0;
	static int nu = 1;
	static char name[20];
	time_t timeofday;
	struct tm *tm;
	
	if ( s == NULL ) {
		s = name;
	}
	if ( seed == 0 ) {
		timeofday = 0;
		timeofday = time(&timeofday);
		tm = localtime(&timeofday);
		seed = tm->tm_sec +60*tm->tm_min+3600*(tm->tm_hour)+1;
		srand(seed);
		rnd = (unsigned int)rand();
		rnd %= 9999;
	}
	sprintf(s,"%05d%03d.tmp",(int)rnd,nu);
	++nu;
	if ( nu >= 1000 ) nu = 1;
	return(s);
}

int
dos_chdir(dir)
char *dir;
{
	int rc;
	char drive_letter;
	unsigned int drive_nu;
	unsigned int logical_drives;
	unsigned int drive;
	unsigned int present_drive;
	
	if ( strlen(dir) <= 2 ) {
		rc = chdir(dir);
		return (rc);
	}

	if ( dir[1] != ':' ) {
		rc = chdir(dir);
		return (rc);
	}

	/* Set drive */ 
	_dos_getdrive(&present_drive);

	drive_letter = dir[0];
	if ( ! isalpha(drive_letter) ) {
		rc = chdir(dir);
		return (rc);
	}
	if ( isupper(drive_letter) ) {
		drive_nu = drive_letter - 'A' + 1;
	} else {
		drive_nu = drive_letter - 'a' + 1;
	}

	_dos_setdrive(drive_nu, &logical_drives);
	_dos_getdrive(&drive);
	if ( drive != drive_nu ) {
		rc = chdir(dir);
		return (rc);
	}
	dir += 2;
	rc = chdir(dir);
	if ( rc != 0 ) {
		_dos_setdrive(present_drive, &logical_drives);
	}
	return (rc);
}
